--------------------------------------------------------------------------------
-- 
-- VHDL Test Bench for module: 
--
-- Executes an exhaustive Test Bench.
--
--------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_signed.all;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;
use work.csa_mod_multiplier_parameters.all;

ENTITY test_csa_mod_mult IS
END test_csa_mod_mult;

ARCHITECTURE behavior OF test_csa_mod_mult IS 

  -- Component Declaration for the Unit Under Test (UUT)
  COMPONENT csa_mod_multiplier is
  PORT (
    x, y, m: in std_logic_vector(K-1 downto 0);
    clk, reset, start: in std_logic; 
    z: out std_logic_vector(K-1 downto 0);
    done: inout std_logic
    );
  END COMPONENT;
  --Inputs
  SIGNAL x, y, m :  std_logic_vector(K-1 downto 0) := (others=>'0');
  SIGNAL clk, reset, start, done: std_logic;
  --Outputs
  SIGNAL z :  std_logic_vector(K-1 downto 0);

  constant DELAY : time := 100 ns;
  constant PERIOD : time := 200 ns;
  constant DUTY_CYCLE : real := 0.5;
  constant OFFSET : time := 0 ns;



BEGIN

  -- Instantiate the Unit Under Test (UUT)
  uut: csa_mod_multiplier PORT MAP(x => x, y => y, m => m,
                          clk => clk, reset => reset, start => start,
                          z => z, done => done);
                          
   PROCESS -- clock process for clk
   BEGIN
    WAIT for OFFSET;
    CLOCK_LOOP : LOOP
       clk <= '0';
       WAIT FOR (PERIOD *(1.0 - DUTY_CYCLE));
       clk <= '1';
       WAIT FOR (PERIOD * DUTY_CYCLE);
    END LOOP CLOCK_LOOP;
   END PROCESS;

  tb_proc : PROCESS --generate values
    VARIABLE m_int: natural;
    VARIABLE TX_LOC : LINE;
    VARIABLE TX_STR : String(1 to 4096);
    BEGIN
      start <= '0'; reset <= '1';
      WAIT FOR PERIOD;
      reset <= '0';
      WAIT FOR PERIOD;
      m_int := 239;
      m <= CONV_STD_LOGIC_VECTOR (m_int, 8);
      for J in 0 to 238 loop
        for I in 0 to 238 loop
          x <= CONV_STD_LOGIC_VECTOR (I, 8);
          y <= CONV_STD_LOGIC_VECTOR (J, 8);
          start <= '1';
          WAIT FOR PERIOD;
          start <= '0';
          wait until done = '1';
          WAIT FOR PERIOD;
          IF ( ((I*J) mod m_int) /= ieee.std_logic_unsigned.CONV_INTEGER(z) ) THEN 
            write(TX_LOC,string'("ERROR!!! X=")); write(TX_LOC, x);
            write(TX_LOC,string'("* Y=")); write(TX_LOC, y);
            write(TX_LOC,string'(" mod M=")); write(TX_LOC, m);
            write(TX_LOC,string'(" is Z=")); write(TX_LOC, z);
            write(TX_LOC,string'(" instead of:")); write(TX_LOC, (I*J) mod m_int);
            write(TX_LOC, string'(" "));
            write(TX_LOC,string'(" (i=")); write(TX_LOC, i);
            write(TX_LOC,string'(" j=")); write(TX_LOC, j); 
            write(TX_LOC, string'(")"));
            TX_STR(TX_LOC.all'range) := TX_LOC.all;
            Deallocate(TX_LOC);
            ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
          END IF;  
          end loop;
      end loop;
    WAIT FOR DELAY;
    ASSERT (FALSE) REPORT
    "Simulation successful (not a failure).  No problems detected. "
    SEVERITY FAILURE;
   END PROCESS;

END;
